/* ************************************************************************
*  file: act.offensive.c , Implementation of commands.    Part of DIKUMUD *
*  Usage : Offensive commands.                                            *
*  Copyright (C) 1990, 1991 - see 'license.doc' for complete information. *
************************************************************************* */

#include <stdio.h>
#include <strings.h>

#include "structs.h"
#include "utils.h"
#include "comm.h"
#include "interpreter.h"
#include "handler.h"
#include "db.h"
#include "spells.h"
#include "limits.h"

/* extern variables */

extern struct room_data *world;
extern struct descriptor_data *descriptor_list;
extern struct room_data *world;


void raw_kill(struct char_data *ch);
void check_killer(struct char_data *ch, struct char_data *victim);
bool is_in_safe(struct char_data *ch, struct char_data *victim);
bool is_first_level(struct char_data *ch, struct char_data *victim);
bool nokill(struct char_data *ch, struct char_data *victim);

/* Nokill--user can toggle this option so they don't inadvertently
get the flag for killing a player. Swiftest.
*/

bool nokill(struct char_data *ch, struct char_data *victim)
{
	if((ch->specials.nokill==TRUE)&&!IS_NPC(ch)&&!IS_NPC(victim)&&
	   (ch!=victim)&&!(IS_SET(world[ch->in_room].room_flags,ARENA))){
		if(victim->specials.fighting==ch){ /* NOKILL ignored if attacked */
			return(FALSE);
		}
		send_to_char("*** Turn NOKILL off first ***",ch);
		return(TRUE);
	}
	return(FALSE);
}
		
/* Do not allow a first level char to attack another player until 2nd lv */
/* and do not allow someone to kill a first level player */
bool is_first_level(struct char_data *ch, struct char_data *victim)
{
	if (IS_NPC(victim)||IS_NPC(ch)){
		return FALSE;
	}
	/* No rule exists for NPCs.. only PCs should get to next statement */
	if ((!IS_NPC(victim))&&(GET_EXP(ch)<1500)){
		send_to_char("You may not attack a player until you have at least 1,500 exp!\r\n",ch);
		return TRUE;
	} else if ( (GET_LEVEL(ch)>1)&&(GET_LEVEL(victim)==1) ){
		send_to_char("You may not attack a first level player.\r\n",ch);
		return TRUE;
	} else {
		return FALSE;
	}
}

bool is_in_safe(struct char_data *ch, struct char_data *victim)
	/* checks to see if PC is in safe room*/
{

	if(IS_NPC(ch)||IS_NPC(victim)){
		return FALSE;
	}
	if(IS_SET(world[ch->in_room].room_flags,SAFE)){
		send_to_char("No fighting permitted in this room.\r\n",ch);
		return TRUE;
	} else {
		return FALSE;
	}
}

void do_hit(struct char_data *ch, char *argument, int cmd)
{
	char arg[MAX_STRING_LENGTH];
	char buffer[MAX_STRING_LENGTH];
	struct char_data *victim;

	one_argument(argument, arg);

	if (*arg) {
		victim = get_char_room_vis(ch, arg);
		if (victim) {
			if (!IS_NPC(victim)){
				send_to_char("You must MURDER a player.\r\n",ch);
				return;
			}
			if (victim == ch) {
				send_to_char("You hit yourself..OUCH!.\r\n", ch);
				act("$n hits $mself, and says OUCH!", FALSE, ch, 0, victim, TO_ROOM);
			} else {
				if (is_in_safe(ch,victim)==TRUE){
					return;
				}

				if (is_first_level(ch,victim)==TRUE){
					return;
				}

				if (IS_AFFECTED(ch, AFF_CHARM) && (ch->master == victim)) {
					act("$N is just such a good friend, you simply can't hit $M.",
					  FALSE, ch,0,victim,TO_CHAR);
					return;
				}
				if ((GET_POS(ch)==POSITION_STANDING) &&
				    (victim != ch->specials.fighting)) {
					hit(ch, victim, TYPE_UNDEFINED);
					WAIT_STATE(ch, PULSE_VIOLENCE+2); /* HVORFOR DET?? */
				} else {
					send_to_char("You do the best you can!\r\n",ch);
				}
			}
		} else {
			send_to_char("They aren't here.\r\n", ch);
		}
	} else {
		send_to_char("Hit whom?\r\n", ch);
	}
}


void do_murder(struct char_data *ch, char *argument, int cmd)
{
	char arg[MAX_STRING_LENGTH];
	char buffer[MAX_STRING_LENGTH];
	struct char_data *victim;

	one_argument(argument, arg);

	if (*arg) {
		victim = get_char_room_vis(ch, arg);
		if (victim) {
			if (victim == ch) {
				send_to_char("You hit yourself..OUCH!.\r\n", ch);
				act("$n hits $mself, and says OUCH!", FALSE, ch, 0, victim, TO_ROOM);
			} else {
				if(is_in_safe(ch,victim)==TRUE){ return; }
				if(is_first_level(ch,victim)==TRUE){ return; }
				if(nokill(ch,victim)==TRUE){ return; }

				if (IS_AFFECTED(ch, AFF_CHARM) && (ch->master == victim)) {
					act("$N is just such a good friend, you simply can't hit $M.",
					  FALSE, ch,0,victim,TO_CHAR);
					return;
				}
				if ((GET_POS(ch)==POSITION_STANDING) &&
				    (victim != ch->specials.fighting)) {
					hit(ch, victim, TYPE_UNDEFINED);
					WAIT_STATE(ch, PULSE_VIOLENCE+2); /* HVORFOR DET?? */
				} else {
					send_to_char("You do the best you can!\r\n",ch);
				}
			}
		} else {
			send_to_char("They aren't here.\r\n", ch);
		}
	} else {
		send_to_char("Hit whom?\r\n", ch);
	}
}

void do_kill(struct char_data *ch, char *argument, int cmd)
{
	static char arg[MAX_STRING_LENGTH];
	char buf[70];
	struct char_data *victim;

	if (GET_LEVEL(ch) < LV_GOD || IS_NPC(ch)) {
		do_hit(ch, argument, 0);
    		return;
	}

	one_argument(argument, arg);

	if (!*arg)
	{
		send_to_char("Slay whom?\r\n", ch);
	}
	else
	{
		if (!(victim = get_char_room_vis(ch, arg)))
   		send_to_char("He/she isn't here.\r\n", ch);
		else
   		if (ch == victim)
      	send_to_char("Your mother would be so sad.. :(\r\n", ch);
   		else {
				act("You chop $M to pieces! Ah! The blood!", FALSE, ch, 0, victim, TO_CHAR);
				act("$N chops you to pieces!", FALSE, victim, 0, ch, TO_CHAR);
				act("$n brutally slays $N", FALSE, ch, 0, victim, TO_NOTVICT);
				raw_kill(victim);
			}
	}
}



void do_backstab(struct char_data *ch, char *argument, int cmd)
{
	struct char_data *victim;
	char name[256];
	byte percent;

	one_argument(argument, name);


	if (!(victim = get_char_room_vis(ch, name))) {
		send_to_char("Backstab who?\r\n", ch);
		return;
	}
	
	if (victim == ch) {
		send_to_char("How can you sneak up on yourself?\r\n", ch);
		return;
	}

	if(is_in_safe(ch,victim)==TRUE){ return; }
	if(is_first_level(ch,victim)==TRUE){ return; }
	if(nokill(ch,victim)==TRUE){ return; }

	if (!ch->equipment[WIELD]) {
		send_to_char("You need to wield a weapon, to make it a succes.\r\n",ch);
		return;
	}

	if (ch->equipment[WIELD]->obj_flags.value[3] != 11) {
		send_to_char("Only piercing weapons can be used for backstabbing.\r\n",ch);
		return;
	}

	if (victim->specials.fighting) {
		send_to_char("You can't backstab a fighting person, too alert!\r\n", ch);
		return;
	}

	percent=number(1,101); /* 101% is a complete failure */

	if (AWAKE(victim) && (percent > ch->skills[SKILL_BACKSTAB].learned))
		damage(ch, victim, 0, SKILL_BACKSTAB);
	else
		hit(ch,victim,SKILL_BACKSTAB);
}



void do_order(struct char_data *ch, char *argument, int cmd)
{
  char name[100], message[256];
	char buf[256];
	bool found = FALSE;
	int org_room;
	struct char_data *victim;
	struct follow_type *k;

  half_chop(argument, name, message);

	if (!*name || !*message)
		send_to_char("Order who to do what?\r\n", ch);
	else if (!(victim = get_char_room_vis(ch, name)) &&
	         str_cmp("follower", name) && str_cmp("followers", name))
			send_to_char("That person isn't here.\r\n", ch);
	else if (ch == victim)
		send_to_char("You obviously suffer from schitzophrenia.\r\n", ch);

	else {
		if (IS_AFFECTED(ch, AFF_CHARM)) {
			send_to_char("Your superior would not aprove of you giving orders.\r\n",ch);
			return;
		}

		if (victim) {
			sprintf(buf, "$N orders you to '%s'", message);
			act(buf, FALSE, victim, 0, ch, TO_CHAR);
			act("$n gives $N an order.", FALSE, ch, 0, victim, TO_ROOM);

			if ( (victim->master!=ch) || !IS_AFFECTED(victim, AFF_CHARM) )
				act("$n has an indifferent look.", FALSE, victim, 0, 0, TO_ROOM);
			else {
				send_to_char("Ok.\r\n", ch);
				command_interpreter(victim, message);
			}
		} else {  /* This is order "followers" */
			sprintf(buf, "$n issues the order '%s'.", message);
			act(buf, FALSE, ch, 0, victim, TO_ROOM);

			org_room = ch->in_room;

			for (k = ch->followers; k; k = k->next) {
				if (org_room == k->follower->in_room)
					if (IS_AFFECTED(k->follower, AFF_CHARM)) {
						found = TRUE;
						command_interpreter(k->follower, message);
					}
			}
			if (found)
				send_to_char("Ok.\r\n", ch);
			else
				send_to_char("Nobody here is a loyal subject of yours!\r\n", ch);
		}
	}
}



void do_flee(struct char_data *ch, char *argument, int cmd)
{
	int i, attempt, loose, die;

	void gain_exp(struct char_data *ch, int gain);
	int special(struct char_data *ch, int cmd, char *arg);

	if (!(ch->specials.fighting)) {
		for(i=0; i<6; i++) {
			attempt = number(0, 5);  /* Select a random direction */
			if (CAN_GO(ch, attempt) &&
			!IS_SET(world[EXIT(ch, attempt)->to_room].room_flags, DEATH)) {
				act("$n panics, and attempts to flee.", TRUE, ch, 0, 0, TO_ROOM);
				if ((die = do_simple_move(ch, attempt, FALSE))== 1) {
					/* The escape has succeded */
					send_to_char("You flee head over heels.\r\n", ch);
				} else {
					if (!die) act("$n tries to flee, but is too exhausted!", TRUE, ch, 0, 0, TO_ROOM);
					return;
				}
			}
		} /* for */
		/* No exits was found */
		send_to_char("PANIC! You couldn't escape!\r\n", ch);
		return;
	}

	for(i=0; i<6; i++) {
		attempt = number(0, 5);  /* Select a random direction */
		if (CAN_GO(ch, attempt) &&
		   !IS_SET(world[EXIT(ch, attempt)->to_room].room_flags, DEATH)) {
			act("$n panics, and attempts to flee.", TRUE, ch, 0, 0, TO_ROOM);
			if ((die = do_simple_move(ch, attempt, FALSE))== 1) { 
              /* The escape has succeded */
				loose = GET_MAX_HIT(ch->specials.fighting)-GET_HIT(ch->specials.fighting);
				loose *= GET_LEVEL(ch->specials.fighting);

				if (!IS_NPC(ch)&&!IS_SET(world[ch->in_room].room_flags,ARENA)) {
					gain_exp(ch, -loose);
				} /* Don't lose exp in the arena */

				send_to_char("You flee head over heels.\r\n", ch);

				/* Insert later when using huntig system        */
				/* ch->specials.fighting->specials.hunting = ch */

				if (ch->specials.fighting->specials.fighting == ch)
					stop_fighting(ch->specials.fighting);
				stop_fighting(ch);
				return;
			} else {
				if (!die) act("$n tries to flee, but is too exhausted!", TRUE, ch, 0, 0, TO_ROOM);
				return;
			}
		}
	} /* for */

	/* No exits was found */
	send_to_char("PANIC! You couldn't escape!\r\n", ch);
}



void do_bash(struct char_data *ch, char *argument, int cmd)
{
	struct char_data *victim;
	char name[256], buf[256];
	byte percent;

	one_argument(argument, name);

	if ((GET_CLASS(ch) != CLASS_WARRIOR) &&
	    (!IS_SET(ch->specials.act,PLR_ISMULTIWA))) {
		send_to_char("You better leave all the martial arts to fighters.\r\n", ch);
		return;
	}

	if (!(victim = get_char_room_vis(ch, name))) {
		if (ch->specials.fighting) {
			victim = ch->specials.fighting;
		} else {
			send_to_char("Bash whom?\r\n", ch);
			return;
		}
	}
	
	if (victim == ch) {
		send_to_char("Aren't we funny today...\r\n", ch);
		return;
	}

	if(is_in_safe(ch,victim)==TRUE) { return; }
	if(is_first_level(ch,victim)==TRUE){ return; }
	if (nokill(ch,victim)==TRUE){ return; }

	if (!ch->equipment[WIELD]) {
		send_to_char("You need to wield a weapon, to make it a success.\r\n",ch);
		return;
	}

	percent=number(1,101); /* 101% is a complete failure */

	if (percent > ch->skills[SKILL_BASH].learned) {
		damage(ch, victim, 0, SKILL_BASH);
		GET_POS(ch) = POSITION_SITTING;
	} else {
		damage(ch, victim, 1, SKILL_BASH);
		GET_POS(victim) = POSITION_SITTING;
		WAIT_STATE(victim, PULSE_VIOLENCE*2);
	}
	WAIT_STATE(ch, PULSE_VIOLENCE*2);
}



void do_rescue(struct char_data *ch, char *argument, int cmd)
{
	struct char_data *victim, *tmp_ch;
	int percent;
	char victim_name[240];
	char buf[240];

	one_argument(argument, victim_name);

	if (!(victim = get_char_room_vis(ch, victim_name))) {
		send_to_char("Who do you want to rescue?\r\n", ch);
		return;
	}

	if (victim == ch) {
		send_to_char("What about fleeing instead?\r\n", ch);
		return;
	}

	if (ch->specials.fighting == victim) {
		send_to_char("How can you rescue someone you are trying to kill?\r\n",ch);
		return;
	}

	for (tmp_ch=world[ch->in_room].people; tmp_ch &&
	    (tmp_ch->specials.fighting != victim); tmp_ch=tmp_ch->next_in_room)  ;

	if (!tmp_ch) {
		act("But nobody is fighting $M?", FALSE, ch, 0, victim, TO_CHAR);
		return;
	}


	if ((GET_CLASS(ch) != CLASS_WARRIOR) &&
	    (!IS_SET(ch->specials.act,PLR_ISMULTIWA)))
		send_to_char("But only true warriors can do this!", ch);
	else {
		percent=number(1,101); /* 101% is a complete failure */
		if (percent > ch->skills[SKILL_RESCUE].learned) {
			send_to_char("You fail the rescue.\r\n", ch);
			return;
		}

		send_to_char("Banzai! To the rescue...\r\n", ch);
		act("You are rescued by $N, you are confused!", FALSE, victim, 0, ch, TO_CHAR);
		act("$n heroically rescues $N.", FALSE, ch, 0, victim, TO_NOTVICT);

		if (victim->specials.fighting == tmp_ch)
			stop_fighting(victim);
		if (tmp_ch->specials.fighting)
			stop_fighting(tmp_ch);
		if (ch->specials.fighting)
			stop_fighting(ch);

		check_killer(ch, tmp_ch); /* so rescuing an NPC who is fighting a PC does not result in the other guy getting killer flag */
		set_fighting(ch, tmp_ch);
		set_fighting(tmp_ch, ch);

		WAIT_STATE(victim, 2*PULSE_VIOLENCE);
	}

}



void do_kick(struct char_data *ch, char *argument, int cmd)
{
	struct char_data *victim;
	char name[256], buf[256];
	byte percent;

	if ((GET_CLASS(ch) != CLASS_WARRIOR) &&
	    (!IS_SET(ch->specials.act,PLR_ISMULTIWA))) {
		send_to_char("You better leave all the martial arts to fighters.\r\n", ch);
		return;
	}

	one_argument(argument, name);

	if (!(victim = get_char_room_vis(ch, name))) {
		if (ch->specials.fighting) {
			victim = ch->specials.fighting;
		} else {
			send_to_char("Kick whom?\r\n", ch);
			return;
		}
	}

	if (victim == ch) {
		send_to_char("Aren't we funny today...\r\n", ch);
		return;
	}

	if(is_in_safe(ch,victim)==TRUE){ return; }
	if(is_first_level(ch,victim)==TRUE){ return; }
	if(nokill(ch,victim)==TRUE){ return; }

	percent=((10-(GET_AC(victim)/10))<<1) + number(1,101); /* 101% is a complete failure */

	if (percent > ch->skills[SKILL_KICK].learned) {
		damage(ch, victim, 0, SKILL_KICK);
	} else {
		damage(ch, victim, GET_LEVEL(ch)>>1, SKILL_KICK);
	}
	WAIT_STATE(ch, PULSE_VIOLENCE*3);
}

